#!/usr/bin/env python3

import sys
import os
import time
import argparse
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu
from biobb_io.api.pdb import pdb
from biobb_model.model.fix_side_chain import fix_side_chain
from biobb_model.model.mutate import mutate
from biobb_md.gromacs.pdb2gmx import pdb2gmx
from biobb_md.gromacs.editconf import editconf
from biobb_md.gromacs.solvate import solvate
from biobb_md.gromacs.grompp import grompp
from biobb_md.gromacs.genion import genion
from biobb_md.gromacs.mdrun import mdrun
from biobb_analysis.gromacs.gmx_rms import gmx_rms
from biobb_analysis.gromacs.gmx_rgyr import gmx_rgyr
from biobb_analysis.gromacs.gmx_energy import gmx_energy
from biobb_analysis.gromacs.gmx_image import gmx_image
from biobb_analysis.gromacs.gmx_trjconv_str import gmx_trjconv_str

# Loading the biobb configuration reader
conf = settings.ConfReader(sys.argv[1])

global_log, _ = fu.get_logs(path=conf.get_working_dir_path())
global_prop = conf.get_prop_dic(global_log=global_log)
global_paths = conf.get_paths_dic()

global_log.info("step1_pdb: Download the initial Structure")
pdb(**global_paths["step1_pdb"], properties=global_prop["step1_pdb"])

global_log.info("step2_fixsidechain: Modeling the missing heavy atoms in the structure side chains")
fix_side_chain(**global_paths["step2_fixsidechain"], properties=global_prop["step2_fixsidechain"])

global_log.info("step3_pdb2gmx: Generate the topology")
pdb2gmx(**global_paths["step3_pdb2gmx"], properties=global_prop["step3_pdb2gmx"])

global_log.info("step4_editconf: Create the solvent box")
editconf(**global_paths["step4_editconf"], properties=global_prop["step4_editconf"])

global_log.info("step5_solvate: Fill the solvent box with water molecules")
solvate(**global_paths["step5_solvate"], properties=global_prop["step5_solvate"])

global_log.info("step6_grompp_genion: Preprocess ion generation")
grompp(**global_paths["step6_grompp_genion"], properties=global_prop["step6_grompp_genion"])

global_log.info("step7_genion: Ion generation")
genion(**global_paths["step7_genion"], properties=global_prop["step7_genion"])

global_log.info("step8_grompp_min: Preprocess energy minimization")
grompp(**global_paths["step8_grompp_min"], properties=global_prop["step8_grompp_min"])

global_log.info("step9_mdrun_min: Execute energy minimization")
mdrun(**global_paths["step9_mdrun_min"], properties=global_prop["step9_mdrun_min"])

global_log.info("step10_energy_min: Compute potential energy during minimization")
gmx_energy(**global_paths["step10_energy_min"], properties=global_prop["step10_energy_min"])

global_log.info("step11_grompp_nvt: Preprocess system temperature equilibration")
grompp(**global_paths["step11_grompp_nvt"], properties=global_prop["step11_grompp_nvt"])

global_log.info("step12_mdrun_nvt: Execute system temperature equilibration")
mdrun(**global_paths["step12_mdrun_nvt"], properties=global_prop["step12_mdrun_nvt"])

global_log.info("step13_energy_nvt: Compute temperature during NVT equilibration")
gmx_energy(**global_paths["step13_energy_nvt"], properties=global_prop["step13_energy_nvt"])

global_log.info("step14_grompp_npt: Preprocess system pressure equilibration")
grompp(**global_paths["step14_grompp_npt"], properties=global_prop["step14_grompp_npt"])

global_log.info("step15_mdrun_npt: Execute system pressure equilibration")
mdrun(**global_paths["step15_mdrun_npt"], properties=global_prop["step15_mdrun_npt"])

global_log.info("step16_energy_npt: Compute Density & Pressure during NPT equilibration")
gmx_energy(**global_paths["step16_energy_npt"], properties=global_prop["step16_energy_npt"])

global_log.info("step17_grompp_md: Preprocess free dynamics")
grompp(**global_paths["step17_grompp_md"], properties=global_prop["step17_grompp_md"])

global_log.info("step18_mdrun_md: Execute free molecular dynamics simulation")
mdrun(**global_paths["step18_mdrun_md"], properties=global_prop["step18_mdrun_md"])

global_log.info("step19_rmsfirst: Compute Root Mean Square deviation against equilibrated structure (first)")
gmx_rms(**global_paths["step19_rmsfirst"], properties=global_prop["step19_rmsfirst"])

global_log.info("step20_rmsexp: Compute Root Mean Square deviation against minimized structure (exp)")
gmx_rms(**global_paths["step20_rmsexp"], properties=global_prop["step20_rmsexp"])

global_log.info("step21_rgyr: Compute Radius of Gyration to measure the protein compactness during the free MD simulation")
gmx_rgyr(**global_paths["step21_rgyr"], properties=global_prop["step21_rgyr"])

global_log.info("step22_image: Imaging the resulting trajectory")
gmx_image(**global_paths["step22_image"], properties=global_prop["step22_image"])

global_log.info("step23_dry: Removing water molecules and ions from the resulting structure")
gmx_trjconv_str(**global_paths["step23_dry"], properties=global_prop["step23_dry"])
